/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2016 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  $Id: tSIOPortGRCitrusMain_inline.h 388 2019-05-22 11:25:18Z coas-nagasima $
 */

/*
 *		シリアルインタフェースドライバのターゲット依存部（GR-CITRUS用）
 */

/*
 *  SIOポートのオープン
 */
Inline void
eSIOPort_open(CELLIDX idx)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);

	/*
	 *  デバイス依存のオープン処理
	 */
	cSIOPort_open();
}

/*
 *  SIOポートのクローズ
 */
Inline void
eSIOPort_close(CELLIDX idx)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);

	/*
	 *  デバイス依存のクローズ処理
	 */
	cSIOPort_close();
}

/*
 *  SIOポートへの文字送信
 */
Inline bool_t
eSIOPort_putChar(CELLIDX idx, char c)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);

	return(cSIOPort_putChar(c));
}

/*
 *  SIOポートからの文字受信
 */
Inline int_t
eSIOPort_getChar(CELLIDX idx)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);

	return(cSIOPort_getChar());
}

/*
 *  SIOポートからのコールバックの許可
 */
Inline void
eSIOPort_enableCBR(CELLIDX idx, uint_t cbrtn)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);

	cSIOPort_enableCBR(cbrtn);
}

/*
 *  SIOポートからのコールバックの禁止
 */
Inline void
eSIOPort_disableCBR(CELLIDX idx, uint_t cbrtn)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);

	cSIOPort_disableCBR(cbrtn);
}

/*
 *  SIOポートからの送信可能コールバック
 */
Inline void
eiSIOCBR_readySend(CELLIDX idx)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);

	if (is_ciSIOCBR_joined()) {
		ciSIOCBR_readySend();
	}
}

/*
 *  SIOポートからの受信通知コールバック
 */
Inline void
eiSIOCBR_readyReceive(CELLIDX idx)
{
	CELLCB	*p_cellcb = GET_CELLCB(idx);
	
	if (is_ciSIOCBR_joined()) {
		ciSIOCBR_readyReceive();
	}
}
